using System;
using System.Runtime.InteropServices;
using PkmnLibSharp.DynamicData;
using PkmnLibSharp.FFI.StaticData;
using PkmnLibSharp.StaticData;

namespace PkmnLibSharp.FFI.DynamicData
{
    internal static class Pokemon
    {
        /// <summary>
        /// Instantiates a new Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_new(IntPtr dynamicLibrary, IntPtr species, IntPtr form,
            byte hiddenAbility, byte abilityIndex, LevelInt level, uint uniqueIdentifier, Gender gender, byte coloring,
            IntPtr natureName);
        
        /// <summary>
        /// Drops an Arc reference held by the FFI.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_drop(IntPtr pokemon);

        /// <summary>
        /// The library data of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_library(IntPtr pokemon);

        /// <summary>
        /// The species of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_species(IntPtr pokemon);

        /// <summary>
        /// The form of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_form(IntPtr pokemon);

        /// <summary>
        /// The species that should be displayed to the user. This handles stuff like the Illusion ability.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_display_species(IntPtr pokemon);

        /// <summary>
        /// The form that should be displayed to the user. This handles stuff like the Illusion ability.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_display_form(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern LevelInt pokemon_level(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern uint pokemon_experience(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern uint pokemon_unique_identifier(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern Gender pokemon_gender(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_coloring(IntPtr pokemon);

        /// <summary>
        /// Gets the held item of a Pokemon
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_held_item(IntPtr pokemon);

        /// <summary>
        /// Checks whether the Pokemon is holding a specific item.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_has_held_item(IntPtr pokemon, IntPtr itemName);

        /// <summary>
        /// Changes the held item of the Pokemon. Returns the previously held item.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_set_held_item(IntPtr pokemon, IntPtr item);

        /// <summary>
        /// Removes the held item from the Pokemon. Returns the previously held item.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_remove_held_item(IntPtr pokemon);

        /// <summary>
        /// Makes the Pokemon uses its held item.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_consume_held_item(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern uint pokemon_current_health(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern uint pokemon_max_health(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern float pokemon_weight(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern float pokemon_height(IntPtr pokemon);

        /// <summary>
        /// An optional nickname of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IntPtr pokemon_nickname(IntPtr pokemon);

        /// <summary>
        /// Whether the actual ability on the form is a hidden ability.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_real_ability_is_hidden(IntPtr pokemon);

        /// <summary>
        /// The index of the actual ability on the form.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_real_ability_index(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern ulong pokemon_types_length(IntPtr pokemon);

        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern TypeIdentifier pokemon_types_get(IntPtr pokemon, ulong index);

        /// <summary>
        /// Gets a learned move of the Pokemon. Index should generally be below [`MAX_MOVES`], you will get
        /// a null pointer otherwise.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_learned_move_get(IntPtr pokemon, ulong index);
        
        /// <summary>
        /// The stats of the Pokemon when disregarding any stat boosts.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_flat_stats(IntPtr pokemon);

        /// <summary>
        /// The stats of the Pokemon including the stat boosts.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_boosted_stats(IntPtr pokemon);

        /// <summary>
        /// Get the stat boosts for a specific stat.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern sbyte pokemon_get_stat_boost(IntPtr pokemon, Statistic statistic);

        /// <summary>
        /// Change a boosted stat by a certain amount.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern sbyte pokemon_change_stat_boost(IntPtr pokemon, Statistic statistic, sbyte diffAmount,
            byte selfInflicted);
        
        /// <summary>
        /// Gets a <a href="https://bulbapedia.bulbagarden.net/wiki/Individual_values">individual value</a> of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_get_individual_value(IntPtr pokemon, Statistic statistic);

        /// <summary>
        /// Modifies a <a href="https://bulbapedia.bulbagarden.net/wiki/Individual_values">individual value</a> of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_set_individual_value(IntPtr pokemon, Statistic statistic, byte value);

        /// <summary>
        /// Gets a <a href="https://bulbapedia.bulbagarden.net/wiki/Effort_values">effort value</a> of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_get_effort_value(IntPtr pokemon, Statistic statistic);

        /// <summary>
        /// Modifies a <a href="https://bulbapedia.bulbagarden.net/wiki/Effort_values">effort value</a> of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_set_effort_value(IntPtr pokemon, Statistic statistic, byte value);

        /// <summary>
        /// Gets the data for the battle the Pokemon is currently in. If the Pokemon is not in a battle, this
        /// returns null.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_get_battle(IntPtr pokemon);

        /// <summary>
        /// Get the index of the side of the battle the Pokemon is in. If the Pokemon
        /// is not on the battlefield, this always returns 0.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_get_battle_side_index(IntPtr pokemon);

        /// <summary>
        /// Get the index of the slot on the side of the battle the Pokemon is in. If the Pokemon
        /// is not on the battlefield, this always returns 0.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_get_battle_index(IntPtr pokemon);

        /// <summary>
        /// Returns whether something overrides the ability.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_is_ability_overriden(IntPtr pokemon);

        /// <summary>
        /// Returns the currently active ability.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_active_ability(IntPtr pokemon);

        /// <summary>
        /// Whether or not the Pokemon is allowed to gain experience.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_allowed_experience_gain(IntPtr pokemon);

        /// <summary>
        /// The <a href="https://bulbapedia.bulbagarden.net/wiki/Nature">nature</a> of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern IdentifiablePointer pokemon_nature(IntPtr pokemon);

        /// <summary>
        /// Calculates the flat stats on the Pokemon. This should be called when for example the base
        /// stats, level, nature, IV, or EV changes. This has a side effect of recalculating the boosted
        /// stats, as those depend on the flat stats.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_recalculate_flat_stats(IntPtr pokemon);

        /// <summary>
        /// Calculates the boosted stats on the Pokemon. This should be called when a stat boost changes.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_recalculate_boosted_stats(IntPtr pokemon);

        /// <summary>
        /// Change the species of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_change_species(IntPtr pokemon, IntPtr species, IntPtr form);

        /// <summary>
        /// Change the form of the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_change_form(IntPtr pokemon, IntPtr form);

        /// <summary>
        /// Whether or not the Pokemon is usable in a battle.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_is_usable(IntPtr pokemon);

        /// <summary>
        /// Returns whether the Pokemon is fainted.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_is_fainted(IntPtr pokemon);

        /// <summary>
        /// Whether or not the Pokemon is on the battlefield.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_is_on_battlefield(IntPtr pokemon);

        /// <summary>
        /// Damages the Pokemon by a certain amount of damage, from a damage source.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_damage(IntPtr pokemon, uint damage, DamageSource damageSource);

        /// <summary>
        /// Heals the Pokemon by a specific amount. Unless allowRevive is set to 1, this will not
        /// heal if the Pokemon has 0 health. If the amount healed is 0, this will return false.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern byte pokemon_heal(IntPtr pokemon, uint damage, byte allowRevive);

        /// <summary>
        /// Learn a move.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_learn_move(IntPtr pokemon, IntPtr moveName, MoveLearnMethod learnMethod);

        /// <summary>
        /// Removes the current non-volatile status from the Pokemon.
        /// </summary>
        [DllImport(Data.DllName, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        internal static extern void pokemon_clear_status(IntPtr pokemon);
    }
}