using System;
using PkmnLibSharp.FFI;
using PkmnLibSharp.Utils;
using Interface = PkmnLibSharp.FFI.StaticData.Libraries.ItemLibrary;

namespace PkmnLibSharp.StaticData.Libraries
{
    public class ItemLibrary : DataLibrary<Item>
    {
        public ItemLibrary(ulong capacity) : base(Interface.item_library_new(capacity), true)
        {
        }

        internal ItemLibrary(IdentifiablePointer ptr, bool isOwner) : base(ptr, isOwner)
        {
        }

        protected override void Destructor() => Interface.item_library_drop(Ptr);

        public override void Add(string key, Item value) =>
            Interface.item_library_add(Ptr, key.ToPtr(), value.TakeOwnershipAndInvalidate());

        public override int Count => (int)Interface.item_library_len(Ptr);

        protected override Item? GetValueByKey(string key)
        {
            var ptr = Interface.item_library_get(Ptr, key.ToPtr());
            return ptr.Ptr == IntPtr.Zero ? null : new Item(ptr);
        }
        
        public override string? GetKeyByIndex(ulong index) =>
            Interface.item_library_get_key_by_index(Ptr, index).PtrString();

        public override void InvalidateChildren()
        {
            foreach (var value in Cache.ValueCache.Values)
            {
                value.Invalidate();
            }
        }
        
        ~ItemLibrary()
        {
            Dispose();
        }
    }
}